// Test code for the bilinear picture scaler.
// Written by Nils Liaaen Corneliusen 2014.
// License: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication license
// Please refer to the article at https://www.ignorantus.com/pages/tilegx_bilinear/ for more information.

#include <stdio.h>
#include <stdlib.h>
#include <arch/cycle.h>
#include <arch/spr.h>

#include "bmp.h"
#include "par.h"
#include "scaler.h"

static void pad_right_lower( bmp_argb *bp )
{
    uint32_t *dst = (uint32_t *)(bp->argb + bp->w*4);
    uint32_t *src = dst-1;
    int str4 = bp->stride>>2;

    for( int y = 0; y < bp->h; y++ ) {
        dst[0] = *src;
        dst[1] = *src;
        src += str4;
        dst += str4;
    }

    uint32_t *dst0 = (uint32_t *)(bp->argb + bp->stride*bp->h);
    uint32_t *dst1 = dst0 + str4;
    src = dst0 - str4; 
    for( int x = 0; x < bp->w; x++ ) {
        *dst0++ = *src;
        *dst1++ = *src++;
    }
}


int main( int argc, char *argv[] )
{
    bmp_argb *src;

    if( argc != 5 ) {
        printf( "Usage: %s <input pic> <output pic> <sizex> <sizey>\n", argv[0] );
        return 1;
    }

    int rc = par_init( 35 );
    if( rc != 0 ) return rc;

    src = bmp_load( argv[1], false );
    if( src == NULL ) return 1;

    pad_right_lower( src );

    int xs = atoi( argv[3] );
    int ys = atoi( argv[4] );

    bmp_argb *dst = bmp_alloc( xs, ys );
    if( dst == NULL ) {
        bmp_free( src );
        return 1;
    }

    printf( "Scale %d*%d to %d*%d\n\n", src->w, src->h, dst->w, dst->h );

    printf( "            1        2        4        8       16       32       35\n" );

    for( int lines = 2; lines <= 128; lines *= 2 ) {

        printf( "%03d: ", lines ); fflush( stdout );

        for( int cores2 = 1; cores2 < 128; cores2 *= 2 ) {
            int cores = cores2 == 64 ? 35 : cores2;

            par_set_cores( cores );

            uint64_t t0 = get_cycle_count();
            for( int i = 0; i < cores*30; i++ )
                scale_bilinear_parallel_2pass( src, dst, lines );
            uint64_t t1 = get_cycle_count();

            uint64_t frametime = (t1-t0)/(cores*30);

            printf( "%08lld ", (long long int)frametime ); fflush( stdout );
        }
        printf( "\n" );
    }

    bmp_save( dst, argv[2], false );

    bmp_free( src );
    bmp_free( dst );

    return 0;
}
